#ifndef ARENA_H
#define ARENA_H

#define ARENA_INITIAL_COMMIT_SIZE sizeof(struct Arena)
#define ARENA_DEFAULT_ALIGNMENT   0

#ifndef ARENA_DEFAULT_RESERVE_SIZE
#  define ARENA_DEFAULT_RESERVE_SIZE KB(4)
#endif // ARENA_DEFAULT_RESERVE_SIZE

typedef struct Arena Arena;
struct Arena {
  struct Arena *current;
  struct Arena *prev;
  u64 *backing_buffer;
  u64 base_pos;
  u64 offset;
  u64 size;
  b8 growable;
};

//= rhjr: arenas

internal Arena * arena_initialize(u64 size, b8 growable);
internal Arena * arena_initialize_default();

internal void *  arena_allocate(Arena *arena, u64 size);

#define arena_push_array(arena, type, count) \
  (type*) arena_allocate((arena), sizeof(type) * (count))

#define arena_push(arena, type) \
  (type*) arena_allocate((arena), sizeof(type))

internal void arena_release(Arena *arena);

//= rhjr: memory helpers

#define memory_align(x, b) ((b != 0) ? (((x) + (b) - 1) & (~(b - 1))) : (x))

#define memory_zero(s,z)       memset((s), 0, (z))
#define memory_zero_struct(s)  memory_zero((s), sizeof(*(s)))

#define memory_copy(dst, src, length) memmove((dst), (src), (length))

#endif // ARENA_H
