
internal Key
hash_store_hash(String8 data)
{
  Key result = {0};

  u64 hash = 5381;
  for(u64 i = 0; i < data.length; i += 1)
  {
    hash = ((hash << 5) + hash) + data.content[i];
  }

  memory_copy(&result, &hash, sizeof(u64));
  return result;
}

internal Table *
hash_store_initialize(void)
{
  Arena *arena = arena_initialize_default();
  Arena *str8_arena = arena_initialize_default();

  Table *table = arena_push(arena, Table);
  table->arena = arena;
  table->str8_arena = str8_arena;
  table->slots_count = 200;

  table->slots = arena_push_array(arena, TableSlot, table->slots_count);

  return table;
}

internal Key
hash_store_string8_to_key(Table *table, String8 data)
{
  Key result = hash_store_hash(data);
  u64 slot_idx = result.U32[0]%table->slots_count;

  TableNode *node = arena_push(table->arena, TableNode);
  node->data.content = arena_push_array(table->str8_arena, char, data.length);
  node->data.length = data.length;
  node->key = result;

  memory_copy(&node->data.content, &data.content, data.length);

  TableSlot *slot = &table->slots[slot_idx];
  queue_push(slot->first, slot->last, node);

  return result;
}

internal String8
hash_store_string8_from_key(Table *table, Key key)
{
  String8 result = {0};
  u64 slot_idx = key.U32[0]%table->slots_count;
  TableSlot *slot = &table->slots[slot_idx];

  for (TableNode *n = slot->first; n != 0; n = n->next)
  {
    if (key.U32[1] == n->key.U32[1])
    {
      result = n->data;
      break;
    }
  }

  return result;
}
